#!/usr/bin/python3
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Parser for wbo2.

https://www.wbo2.com/sw/logger.htm Frame and byte info.
"""
import multiprocessing
import time
from typing import Dict
from typing import Generator
from typing import Text

import serial
from absl import app
from absl import flags
from absl import logging

from exit_speed import gps_pb2
from exit_speed import sensor

FLAGS = flags.FLAGS
flags.DEFINE_float('stoichiometric', 14.7,
                   'This is used to convert the Lambda_16 bytes into '
                   'an A/F ratio. This should be changed based on fuel.'
                   'Petrol 14.7, LGP 15.5, Methanol 6.4, Diesel 14.5')
flags.DEFINE_integer('cylinders', 6,
                     'Number of cylinders in the engine.')
# http://techedge.com.au/vehicle/wbo2/wblambda.htm

FRAME_SIZE = 28
# {'frame_type': (slice_left, slice_right)}
# pylint: disable=bad-continuation
# pylint: disable=bad-whitespace
FRAME_FORMAT = {
  'header':         (0, 2),    # Bytes 1 & 2
  'sequence':       (2, 3),    # Byte 3
  'tick':           (3, 5),    # Bytes 4 & 5
  'lambda_16':      (5, 7),    # Bytes 6 & 7
  'ipx':            (7, 9),    # Bytes 8 & 9
  'user_1':         (9, 11),   # Bytes 10 & 11
  'user_2':         (11, 13),  # Bytes 11 & 13
  'user_3':         (13, 15),  # Bytes 13 & 14
  'thermocouple_1': (15, 17),  # Bytes 16 & 17
  'thermocouple_2': (17, 19),  # Bytes 18 & 19
  'thermocouple_3': (19, 21),  # Bytes 20 & 21
  'thermistor':     (21, 23),  # Bytes 22 & 23
  'rpm_count':      (23, 25),  # Bytes 24 & 25
  'status':         (25, 27),  # Bytes 26 & 27
  'crc':            (27, 28),  # Byte 28
}
# pylint: enable=bad-continuation
# pylint: enable=bad-whitespace


def FindFrameStart(ser) -> bytes:
  """Find the frame's header start bytes based on the terminal stream."""
  while True:
    header_byte_1 = None
    header_byte_2 = None
    byte = ser.read()
    if byte[0] == 0x5a:
      header_byte_1 = byte
    if header_byte_1:
      byte = ser.read()
      if byte[0] == 0xa5:
        header_byte_2 = byte
      else:
        header_byte_1 = None  # Reset, perhaps another byte was set to 0x5a.
    if header_byte_1 and header_byte_2:
      return header_byte_1 + header_byte_2 + ser.read(FRAME_SIZE - 2)


def CheckFrame(frame) -> bool:
  return sum(frame) & 0b11111111 == 0xFF


def ReadSerial(ser) -> Generator[bytes, None, None]:
  while True:
    frame = FindFrameStart(ser)
    if CheckFrame(frame):
      yield frame


def GetBytes(frame: bytes, frame_key: Text) -> float:
  """Converts byte data into something usable."""
  low, high = FRAME_FORMAT[frame_key]
  frame_bytes = frame[low:high]
  if 'lambda_16' == frame_key:
    return Lambda16ToAFR(int.from_bytes(frame_bytes, 'big'))
  if 'rpm_count' == frame_key:
    return RPMCountToRPM(int.from_bytes(frame_bytes, 'big'),
                         FLAGS.cylinders)
  elif 'user' in frame_key:
    return int.from_bytes(frame_bytes, 'big') / 8184 * 5
  elif 'thermocouple' in frame_key:
    return int.from_bytes(frame_bytes, 'big') / 1023 * 5 / 101
  return int.from_bytes(frame_bytes, 'big')


def Lambda16ToAFR(lambda_16: float) -> float:
  # http://techedge.com.au/vehicle/wbo2/wblambda.htm
  # https://www.wbo2.com/sw/lambda-16.htm
  # 14.7 = Unleaded stoichiometric point.
  return ((lambda_16 / 8192) + 0.5) * FLAGS.stoichiometric


def RPMCountToRPM(rpm_count: float, cylinders: int) -> float:
  if rpm_count:
    us_between_pulse = rpm_count * 5
    minute = 60 * 10 ** 6  # 60 seconds > microseconds
    return minute / us_between_pulse / (cylinders / 2)
  return 0


class WBO2(sensor.SensorBase):
  """Interface for the WBO2 wideband lambda/AFR controller."""

  def __init__(self, config: Dict, point_queue: multiprocessing.Queue):
    self._next_cycle = 0
    super().__init__(config, point_queue)

  def Loop(self):
    frequency_hz = int(
        self.config.get('wbo2', {}).get('frequency_hz')) or 10
    with serial.Serial('/dev/ttyUSB0', 19200) as ser:
      for frame in ReadSerial(ser):
        while not self.stop_process_signal.value:
          logging.log_every_n_seconds(logging.DEBUG, 'WBO2 Frame: %s',
                                      10, frame)
          now = time.time()
          if now > self._next_cycle:
            self._next_cycle = now + 1.0 / frequency_hz
            point = gps_pb2.Point()
            for frame_key, point_value in self.config['wbo2'].items():
              if frame_key in FRAME_FORMAT:
                setattr(point, point_value, GetBytes(frame, frame_key))
            self.AddPointToQueue(point)


def main(unused_argv):
  with serial.Serial('/dev/ttyUSB0', 19200) as ser:
    for frame in ReadSerial(ser):
      print(GetBytes(frame, 'user_3'))


if __name__ == '__main__':
  app.run(main)



#!/usr/bin/python3
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Unitests for Labjack."""
import multiprocessing
import os
import unittest

import mock
import u3
from absl.testing import absltest

from exit_speed import config_lib
from exit_speed import gps_pb2
from exit_speed import labjack


class TestLabjack(unittest.TestCase):
  """Unitests for Labjack."""

  def setUp(self):
    super().setUp()
    config = config_lib.LoadConfig(
        os.path.join(os.path.dirname(os.path.abspath(__file__)),
            'testdata/test_labjack_config.yaml'))
    self.point_queue = multiprocessing.Queue()
    self.labjack = labjack.Labjack(
        config, self.point_queue, start_process=False)
    self.mock_u3 = mock.create_autospec(u3.U3)
    self.labjack.u3 = self.mock_u3

  def testReadValues(self):
    # pylint: disable=invalid-name
    # pylint: disable=unused-argument
    def _binaryToCalibratedAnalogVoltage(result, isLowVoltage, channelNumber):
      if channelNumber in self.labjack.HIGH_VOLTAGE_CHANNELS:
        self.assertFalse(isLowVoltage)
      else:
        self.assertTrue(isLowVoltage)
      mapping = {32816: 1.5,
                 35696: 2.7,
                 32827: 3.9,
                 39968: 1.4}
      return mapping[result]
    # pylint: enable=invalid-name
    # pylint: enable=unused-argument
    self.mock_u3.getTemperature.return_value = 298.73988991230726
    self.mock_u3.getFeedback.side_effect = [[32816], [35696], [32827], [39968]]
    self.mock_u3.binaryToCalibratedAnalogVoltage.side_effect = (
        _binaryToCalibratedAnalogVoltage)
    self.labjack.ReadValues()
    point = gps_pb2.Point().FromString(self.point_queue.get())
    self.assertEqual(78.061801842153101916, point.labjack_temp_f)
    self.assertEqual(1.5, point.fuel_level_voltage)
    self.assertEqual(2.7, point.water_temp_voltage)
    self.assertEqual(3.9, point.oil_pressure_voltage)
    self.assertEqual(14.0, point.battery_voltage)


if __name__ == '__main__':
  absltest.main()



#!/usr/bin/python3
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Unitests for tire_temperature.py"""
import multiprocessing
import time
import unittest

import mock
from absl.testing import absltest
from mlx import mlx90640

from exit_speed import gps_pb2
from exit_speed import tire_temperature

def _CreateRawFrame():
  frame = []
  for _ in range(32 * 24):
    frame.append(30.0)
  return frame

def _CreateFormattedFrame():
  formatted_frame = []
  for _ in range(32):
    row = []
    temp = 0
    for _ in range(24):
      row.append(temp)
      temp += 10
    formatted_frame.append(row)
  return formatted_frame


class MockMlx9064xSensorBase(unittest.TestCase):

  def setUp(self):
    super().setUp()
    patch = mock.patch.object(mlx90640, 'Mlx9064x')
    self.addCleanup(patch.stop)
    self.mock_mlx = patch.start()


class TestInfraRedSensor(MockMlx9064xSensorBase):
  """Infrared Sensor unittests."""

  def setUp(self):
    super().setUp()
    self.sensor = tire_temperature.InfraRedSensor()

  def testReadFrame(self):
    self.mock_mlx.read_frame.return_value = _CreateRawFrame()
    self.sensor.ReadFrame()

  def testFormatFrame(self):
    raw_frame = _CreateRawFrame()
    formatted_frame = self.sensor.FormatFrame(raw_frame)
    row_count = 0
    for row in formatted_frame:
      row_count += 1
      self.assertEqual(32, len(row))
    self.assertEqual(24, row_count)


class TestTireSensor(MockMlx9064xSensorBase):
  """Tire Temperature unittests."""

  def setUp(self):
    super().setUp()
    self.sensor = tire_temperature.TireSensor()

  def testGetTempsByColumn(self):
    expected = {}
    col_index = 0
    for _ in range(32):
      temp = 0
      temps = []
      for _ in range(24):
        temps.append(temp)
        temp += 10
      expected[col_index] = temps
      col_index += 1
    with mock.patch.object(self.sensor, 'FormatFrame') as mock_format_frame:
      mock_format_frame.return_value = _CreateFormattedFrame()
      self.assertDictEqual(expected, self.sensor.GetTempsByColumn())

  def testGetMedianColumnTemps(self):
    expected = {}
    col_index = 0
    for _ in range(32):
      expected[col_index] = 115.0
      col_index += 1
    with mock.patch.object(self.sensor, 'FormatFrame') as mock_format_frame:
      mock_format_frame.return_value = _CreateFormattedFrame()
      self.assertDictEqual(expected, self.sensor.GetMedianColumnTemps())

  def testGetTireTemps(self):
    column_temp = {
        # Air
        0: 80.0,
        1: 80.0,
        2: 80.0,
        3: 80.0,
        4: 80.0,
        5: 80.0,
        6: 80.0,
        7: 80.0,
        8: 80.0,
        9: 135.0,  # Avg temp between tire and ambient air
        # Inside tire
        10: 190.0,
        11: 190.0,
        12: 190.0,
        13: 190.0,
        14: 190.0,
        # Middle Tire
        15: 200.0,
        16: 200.0,
        17: 200.0,
        18: 200.0,
        19: 200.0,
        20: 200.0,
        # Outside Tire
        21: 230.0,
        22: 230.0,
        23: 230.0,
        24: 230.0,
        # Shock heat/spindle
        25: 185.0,  # Avg temp between tire & spindle
        26: 140.0,
        27: 140.0,
        28: 140.0,
        29: 140.0,
        30: 140.0,
        31: 140.0,
    }
    expected = (190.0, 200.0, 230.0)
    with mock.patch.object(self.sensor, 'GetMedianColumnTemps') as mock_col:
      mock_col.return_value = column_temp
      self.assertTupleEqual(expected, self.sensor.GetTireTemps())


class TestClientServer(unittest.TestCase):
  """Tests the ability for the Pi zero client to send data to the server."""

  def testClientServer(self):
    mock_sensor = mock.create_autospec(tire_temperature.TireSensor)
    mock_sensor.GetTireTemps.return_value = (87.0, 93.0, 98)
    point_queue = multiprocessing.Queue()
    self.server = tire_temperature.TireSensorServer(
        'lf_tire_temp', '127.0.0.1', 27001, {}, point_queue)
    with mock.patch.object(tire_temperature, 'TireSensor') as mock_tire_sensor:
      mock_tire_sensor.return_value = mock_sensor
      client = tire_temperature.TireSensorClient('127.0.0.1', 27001)
    time.sleep(3)
    client.ReadAndSendData()
    time.sleep(3)
    point = gps_pb2.Point().FromString(point_queue.get())
    self.assertEqual(point.lf_tire_temp.inner, 188.6)
    self.assertEqual(point.lf_tire_temp.middle, 199.4)
    self.assertEqual(point.lf_tire_temp.outer, 208.4)


if __name__ == '__main__':
  absltest.main()



#!/usr/bin/python3
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""ExitSpeed unittest."""
import os
import sys
import unittest

import fake_rpi
import mock
import psycopg2
import pytz
from absl import flags
from absl.testing import absltest

from exit_speed import gps_pb2
from exit_speed import timescale
# pylint: disable=wrong-import-position
sys.modules['RPi'] = fake_rpi.RPi     # Fake RPi
sys.modules['RPi.GPIO'] = fake_rpi.RPi.GPIO # Fake GPIO
sys.modules['smbus'] = fake_rpi.smbus # Fake smbus (I2C)
# Fixes adafruit_adxl34x import on Travis.
import adafruit_platformdetect
with mock.patch.object(adafruit_platformdetect, 'Detector') as mock_detector:
  mock_detector.chip.id.return_value = 'BCM2XXX'
  import adafruit_dotstar
  from exit_speed import main
# pylint: enable=wrong-import-position

FLAGS = flags.FLAGS
FLAGS.set_default('config_path',
    os.path.join(os.path.dirname(os.path.abspath(__file__)),
        'testdata/test_config.yaml'))
FLAGS.set_default('data_log_path', '/tmp')


class TestExitSpeed(unittest.TestCase):
  """ExitSpeed unittest."""

  def setUp(self):
    super().setUp()
    self._AddMock(adafruit_dotstar, 'DotStar')
    mock_conn = mock.create_autospec(psycopg2.extensions.connection)
    mock_connect = self._AddMock(timescale, 'ConnectToDB')
    mock_connect.return_value = mock_conn

  def _AddMock(self, module, name):
    patch = mock.patch.object(module, name)
    self.addCleanup(patch.stop)
    return patch.start()

  def testGetLogFilePrefix(self):
    point = gps_pb2.Point()
    point.time.FromJsonString(u'2020-05-23T17:47:44.100Z')
    es = main.ExitSpeed()
    expected = '/tmp/Corrado/2020-05-23T17:47:44.100000'
    self.assertEqual(expected, es.GetLogFilePrefix(point, tz=pytz.UTC))

  def testCalculateElapsedValues(self):
    prior_point = gps_pb2.Point()
    prior_point.lat = 12.000000
    prior_point.lon = 23.000000
    prior_point.time.FromJsonString(u'2020-05-23T17:47:44.100Z')
    point = gps_pb2.Point()
    point.lat = 12.000001
    point.lon = 23.000002
    point.time.FromJsonString(u'2020-05-23T17:47:44.200Z')
    es = main.ExitSpeed()
    es.lap = gps_pb2.Lap()
    es.lap.points.extend([prior_point, point])
    es.point = point
    es.ProcessPoint()
    self.assertEqual(prior_point.elapsed_duration_ms, 0)
    self.assertEqual(prior_point.elapsed_distance_m, 0)
    self.assertEqual(point.elapsed_duration_ms, 100)
    self.assertEqual(point.elapsed_distance_m, 0.2430443280901163)

  def testProcessPoint(self):
    prior_point = gps_pb2.Point()
    prior_point.lat = 12.000000
    prior_point.lon = 23.000000
    prior_point.time.FromJsonString(u'2020-05-23T17:47:44.100Z')
    point = gps_pb2.Point()
    point.lat = 12.000001
    point.lon = 23.000002
    point.time.FromJsonString(u'2020-05-23T17:47:44.200Z')
    es = main.ExitSpeed()
    es.lap = gps_pb2.Lap()
    es.lap.points.extend([prior_point, point])
    es.point = point
    es.ProcessPoint()
    self.assertEqual(2856514.6203466402, point.start_finish_distance)

  def testSetLapTime(self):
    es = main.ExitSpeed()
    first_point = gps_pb2.Point()
    first_point.time.FromJsonString(u'2020-05-23T17:47:44.100Z')
    last_point = gps_pb2.Point()
    last_point.time.FromJsonString(u'2020-05-23T17:49:00.100Z')
    session = gps_pb2.Session()
    lap = session.laps.add()
    lap.points.append(first_point)
    lap.points.append(last_point)
    es.lap = lap
    es.session = session
    es.SetLapTime()
    self.assertEqual(76, lap.duration.ToSeconds())
    self.assertEqual(es.leds.best_lap, lap)

  def testCrossStartFinish(self):
    point_a = gps_pb2.Point()
    point_b = gps_pb2.Point()
    point_c = gps_pb2.Point()
    point_b.start_finish_distance = 5.613414540798601
    point_c.start_finish_distance = 8.86833983566463
    point_a.time.FromMilliseconds(1000)
    point_b.time.FromMilliseconds(2000)
    point_c.time.FromMilliseconds(3000)
    point_a.lat = 45.594961
    point_a.lon = -122.694508
    point_b.lat = 45.594988
    point_b.lon = -122.694587
    point_c.lat = 45.595000
    point_c.lon = -122.694638
    session = gps_pb2.Session()
    session.track = 'Portland International Raceway'
    session.start_finish.lat = 45.595015
    session.start_finish.lon = -122.694526
    lap = session.laps.add()
    lap.points.extend([point_a, point_b])
    es = main.ExitSpeed(min_points_per_session=0)
    es.point = point_c
    es.lap = lap
    es.session = session
    es.CrossStartFinish()
    self.assertEqual(2, len(es.session.laps))
    self.assertEqual(2, len(es.session.laps[0].points))
    self.assertEqual(2, len(es.session.laps[1].points))
    self.assertIn(point_a, es.session.laps[0].points)
    self.assertIn(point_b, es.session.laps[0].points)
    self.assertIn(point_b, es.session.laps[1].points)
    self.assertIn(point_c, es.session.laps[1].points)
    self.assertNotIn(point_c, es.session.laps[0].points)

  def testProcessLap(self):
    es = main.ExitSpeed()
    es.point = es.lap.points.add()
    es.ProcessLap()
    self.assertTrue(es.lap.points)

  def testProcessSession(self):
    point = gps_pb2.Point()
    point.speed = 21
    lap = gps_pb2.Lap()
    session = gps_pb2.Session()
    es = main.ExitSpeed()
    es.point = point
    es.session = session
    es.ProcessSession()

    for _ in session.laps:
      for lap_point in lap.points:
        self.assertEqual(point, lap_point)

    point = gps_pb2.Point()
    point.speed = 1
    es.point = point
    es.ProcessSession()


if __name__ == '__main__':
  absltest.main()



#!/usr/bin/python3
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Unitests for accelerometer.py"""
import multiprocessing
import sys
import unittest

import fake_rpi
import mock
from absl.testing import absltest
# pylint: disable=wrong-import-position
# Fixes dotstar import on Travis.
sys.modules['RPi'] = fake_rpi.RPi     # Fake RPi
sys.modules['RPi.GPIO'] = fake_rpi.RPi.GPIO # Fake GPIO
sys.modules['smbus'] = fake_rpi.smbus # Fake smbus (I2C)
import adafruit_platformdetect
with mock.patch.object(adafruit_platformdetect, 'Detector') as mock_detector:
  mock_detector.chip.id.return_value = 'BCM2XXX'
  from exit_speed import accelerometer
  import busio
  import adafruit_fxos8700
# pylint: enable=wrong-import-position


class TestAccelerometer(unittest.TestCase):
  """Accelerometer unittests."""

  def setUp(self):
    super().setUp()
    with mock.patch.object(busio, 'I2C'):
      with mock.patch.object(adafruit_fxos8700, 'FXOS8700'):
        self.accel = accelerometer.Accelerometer()

  def testCorrectValue(self):
    self.assertEqual(9.32704486216904,
        self.accel.CorrectValue('x', 9.924329799999999))
    self.assertEqual(10.836155704720122,
        self.accel.CorrectValue('y', 10.1596894))
    self.assertEqual(9.76962474276722,
        self.accel.CorrectValue('z', 10.260423308799998))

  def testGetGForces(self):
    self.accel.accelerometer.accelerometer = (
        0.3924229, -0.5072783912, 10.29870)
    result = self.accel.GetGForces()
    expected = (0.005457996517639941, -0.016599597585513083, 0.9997570415398239)
    self.assertEqual(expected, result)

  def testCalcPitchAndRoll(self):
    x_gs = 0.02
    y_gs = -0.71
    z_gs = 0.70
    pitch, roll = self.accel.CalcPitchAndRoll(x_gs, y_gs, z_gs)
    expected = (1.15, 45.41)
    self.assertEqual(expected, (round(pitch, 2), round(roll, 2)))

    x_gs = 0.72
    y_gs = -0.03
    z_gs = 0.69
    pitch, roll = self.accel.CalcPitchAndRoll(x_gs, y_gs, z_gs)
    expected = (46.19, 2.49)
    self.assertEqual(expected, (round(pitch, 2), round(roll, 2)))

  def testAccelerometerProcessLoop(self):
    self.accel.accelerometer.accelerometer = (
        0.3924229, -0.5072783912, 10.29870)
    with mock.patch.object(accelerometer, 'Accelerometer') as mock_accel:
      mock_accel.return_value = self.accel
      config = {'accelerometer': {'frequency_hz': 10}}
      point_queue = multiprocessing.Queue()
      proc = accelerometer.AccelerometerProcess(config, point_queue)
      while point_queue.empty():
        pass
      proc.Join()
      self.assertGreaterEqual(point_queue.qsize(), 0)


if __name__ == '__main__':
  absltest.main()



#!/usr/bin/python3
# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Live data dashboard."""
import datetime
import json
import urllib
from typing import Dict
from typing import List
from typing import Text
from typing import Tuple

import dash
import dateutil
import pandas as pd
import plotly.express as px
from absl import app as absl_app
from absl import flags
from dash import dcc
from dash import html
from dash.dependencies import Input
from dash.dependencies import MATCH
from dash.dependencies import Output

from exit_speed.dashboard import queries

FLAGS = flags.FLAGS
flags.DEFINE_bool(
    'debug', True,
    'Set to true to enable auto reloading of Python code at the expense of '
    'high CPU load.')

app = dash.Dash(__name__)
server = app.server
POINTS_COLUMNS = queries.GetPointsColumns()

app.layout = html.Div(
  style={'display': 'grid'},
  children=[
    dcc.Location(id='url', refresh=False),
    dcc.Store(id='max-time'),
    dcc.Interval(id='interval', interval=15 * 1000, n_intervals=-1),
    dcc.Link('Home', href='/'),
    dcc.Slider(
      id='time-window',
      min=1,
      max=60,
      step=5,
      tooltip={'placement': 'bottom', 'always_visible': True},
      marks={
          1:  {'label': '1m'},
          5:  {'label': '5m'},
          10: {'label': '10m'},
          15: {'label': '15m'},
          30: {'label': '30m'},
          60: {'label': '60m'},
      },
    ),
    dcc.Slider(
      id='refresh',
      min=3,
      max=60,
      value=3,
      step=5,
      tooltip={'placement': 'bottom', 'always_visible': True},
      marks={
          3:  {'label': '3s'},
          5:  {'label': '5s'},
          10: {'label': '10s'},
          15: {'label': '15s'},
          30: {'label': '30s'},
          60: {'label': '60s'},
      },
    ),
    dcc.Dropdown(
      id='points-dropdown',
      options=[{'label': i, 'value': i} for i in POINTS_COLUMNS],
      clearable=False,
      multi=True,
    ),
    html.Div(id='graphs'),
  ],
)

@app.callback(
  Output('url', 'href'),
  Input('url', 'href'),
  Input('time-window', 'value'),
  Input('points-dropdown', 'value'),
  Input('refresh', 'value'),
  prevent_initial_call=True,
)
def UpdateURL(href: Text, time_window: int, points: List[Text], refresh: int):
  args = {'time_window': time_window,
          'points': points,
          'refresh': refresh}
  return urllib.parse.urljoin(href, urllib.parse.urlencode(args, doseq=True))


@app.callback(
  Output('time-window', 'value'),
  Output('points-dropdown', 'value'),
  Output('refresh', 'value'),
  Input('url', 'pathname'),
)
def ParseURL(pathname: Text) -> Tuple[int, List[Text], int]:
  # Strip a leading "/" with [1:]
  params = urllib.parse.parse_qs(pathname[1:])
  if params.get('time_window'):
    time_window = int(params.get('time_window')[0])
  else:
    time_window = 15
  if params.get('refresh'):
    refresh = int(params.get('refresh')[0])
  else:
    refresh = 3
  points = params.get(
              'points',
              ['speed', 'rpm'])
  return time_window, points, refresh


def _GetMaxTime(laps_data: pd.DataFrame) -> Text:
  max_time = laps_data['time'].max().to_pydatetime()
  return json.dumps(max_time.isoformat())


@app.callback(
  Output('graphs', 'children'),
  Output('max-time', 'data'),
  Input('time-window', 'value'),
  Input('points-dropdown', 'value'),
)
def UpdateGraph(
    time_window: int, point_values: List[Text]) -> Tuple[List[dcc.Graph], str]:
  now = datetime.datetime.today()
  start_time = now - datetime.timedelta(minutes=time_window)
  if not isinstance(point_values, list):
    point_values = [point_values]
  graphs = []
  laps_data = queries.GetLiveData(start_time, point_values)
  for point_value in point_values:
    graph_type = 'graph'
    fig = px.line(
      laps_data,
      title=point_value,
      x='time',
      y=point_value,
      hover_data=['lap_id', 'lap_number', point_value])
    fig.update_xaxes(showspikes=True)
    fig.update_yaxes(fixedrange=True)
    fig.update_layout(hovermode='x unified')
    graph = dcc.Graph({'type': graph_type,
                       'index': point_values.index(point_value)},
                      figure=fig,
                      style={'display': 'inline-grid', 'width': '50%'})
    graphs.append(graph)
  return graphs, _GetMaxTime(laps_data)


@app.callback(
    Output('interval', 'interval'),
    Input('refresh', 'value'),
)
def SetIntervalRefresh(refresh: int) -> int:
  return refresh * 1000


@app.callback(
  Output({'type': 'graph', 'index': MATCH}, 'extendData'),
  Input({'type': 'graph', 'index': MATCH}, 'id'),
  Input('points-dropdown', 'value'),
  Input('max-time', 'data'),
  Input('interval', 'n_intervals'),
  Input('refresh', 'value'),
  prevent_initial_call=True,
)
def ExtendGraphData(
    graph_id: Dict,
    point_values: List[Text],
    max_time: str,
    n_interval: int,
    refresh: int):
  if n_interval:
    if not isinstance(point_values, list):
      point_values = [point_values]
    max_time = dateutil.parser.parse(json.loads(max_time))
    start_time = max_time + datetime.timedelta(
        seconds=n_interval * refresh)
    point_value = point_values[graph_id['index']]
    lap_data = queries.GetLiveData(start_time, [point_value])
    x_data = lap_data['time'].astype(str).values
    y_data = lap_data[point_value].values
    # Hover data.
    customdata = lap_data[['lap_id', 'lap_number']].values
    return {'updateData': {'x': [x_data],
                           'y': [y_data],
                           'customdata': [customdata]},
            'traceIndices': [0],
            # Makes the assumption data was logged at 10hz.
            'maxPoints': 10 * 60 * refresh}


def main(unused_argv):
  app.run_server(debug=FLAGS.debug)


if __name__ == '__main__':
  absl_app.run(main)



#!/usr/bin/python3
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Used to cleanup database entries created during testing."""
from absl import app
from absl import flags
from absl import logging

from exit_speed import timescale

FLAGS = flags.FLAGS
flags.DEFINE_integer('min_lap_duration_ms', 60 * 1000,
                     'Nukes laps with duration short than this value.')
flags.DEFINE_integer('max_lap_duration_ms', 60 * 1000 * 3,  # 3 mins.
                     'Nukes laps with duration short than this value.')


def NukeNonLiveData(conn):
  """Delete non live data which usually generated during testing."""
  with conn.cursor() as cursor:
    nuke_statement = """
    DELETE FROM points
    WHERE session_id IN (SELECT id FROM sessions WHERE live_data = False);
    DELETE FROM laps
    WHERE session_id IN (SELECT id FROM sessions WHERE live_data = False);
    DELETE FROM sessions
    WHERE id IN (SELECT id FROM sessions WHERE live_data = False);
    """
    logging.info(nuke_statement)
    cursor.execute(nuke_statement)


def NukeLapsWithNoDuration(conn):
  """Delete any laps without a duration.

  These are usually points logged post session in the paddock.

  Args:
    conn: A connection to the timescale backend.
  """
  with conn.cursor() as cursor:
    nuke_statement = """
    DELETE FROM points
    WHERE lap_id IN (SELECT id FROM laps WHERE duration_ms is NULL);
    """
    logging.info(nuke_statement)
    cursor.execute(nuke_statement)
    nuke_statement = """
    DELETE FROM laps
    WHERE duration_ms is NULL;
    """
    logging.info(nuke_statement)
    cursor.execute(nuke_statement)


def NukeLapsByDuration(conn):
  """Delete laps based on time.  These are usually traffic or the out lap."""
  with conn.cursor() as cursor:
    nuke_statement = """
    DELETE FROM points
    WHERE lap_id IN (SELECT id FROM laps
                     WHERE duration_ms < %s or duration_ms > %s);
    DELETE FROM laps
    WHERE duration_ms < %s OR duration_ms > %s;
    """
    logging.info(nuke_statement)
    args = (FLAGS.min_lap_duration_ms, FLAGS.max_lap_duration_ms,
            FLAGS.min_lap_duration_ms, FLAGS.max_lap_duration_ms)
    cursor.execute(nuke_statement, args)


def NukeHangingLaps(conn):
  """Based on prior deletes these cleans up any laps without points."""
  with conn.cursor() as cursor:
    select_statement = """
    SELECT DISTINCT(lap_id) FROM points
    """
    cursor.execute(select_statement)
    laps_to_keep = set(cursor.fetchall())
    select_statement = """
    SELECT id FROM laps
    """
    cursor.execute(select_statement)
    all_lap_ids = set(cursor.fetchall())
    hanging_lap_ids = all_lap_ids.difference(laps_to_keep)
    nuke_statement = """
    DELETE FROM laps
    WHERE id IN %s
    """
    logging.info(nuke_statement)
    if hanging_lap_ids:
      args = (tuple(hanging_lap_ids),)
      cursor.execute(nuke_statement, args)


def NukeHangingSessions(conn):
  """Based on prior deletes these cleans up any sessions without laps."""
  with conn.cursor() as cursor:
    select_statement = """
    SELECT DISTINCT(session_id) FROM laps
    """
    cursor.execute(select_statement)
    sessions_to_keep = set(cursor.fetchall())
    select_statement = """
    SELECT id FROM sessions
    """
    cursor.execute(select_statement)
    all_session_ids = set(cursor.fetchall())
    hanging_session_ids = all_session_ids.difference(sessions_to_keep)
    nuke_statement = """
    DELETE FROM sessions
    WHERE id IN %s
    """
    logging.info(nuke_statement)
    if hanging_session_ids:
      args = (tuple(hanging_session_ids),)
      cursor.execute(nuke_statement, args)


def CleanupTimescale():
  logging.info('Cleaning up Timescale')
  with timescale.ConnectToDB() as conn:
    NukeNonLiveData(conn)
    NukeLapsWithNoDuration(conn)
    NukeLapsByDuration(conn)

    # Hanging deletions should probably come last.
    NukeHangingLaps(conn)
    NukeHangingSessions(conn)
    conn.commit()


def main(unused_argv):
  CleanupTimescale()


if __name__ == '__main__':
  app.run(main)



# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Library for requesting secrets."""
import json


def GetSecret(secret_id):
  # This module is difficult to get working on a Pi so let's load it only on
  # App Engine for now.
  # pylint: disable=import-outside-toplevel
  from google.cloud import secretmanager
  client = secretmanager.SecretManagerServiceClient()
  response = client.access_secret_version(request={'name': secret_id})
  return json.loads(response.payload.data)



#!/usr/bin/python3
# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Track list and helper functions."""
import attr

@attr.s(frozen=True)
class Track(object):
  name = attr.ib(type=str)
  start_finish = attr.ib(type=tuple)
  turns = attr.ib(type=tuple)


@attr.s(frozen=True)
class Turn(object):
  number = attr.ib(type=str)
  lat = attr.ib(type=float)
  lon = attr.ib(type=float)
  report_range = attr.ib(type=int, default=200)



# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Dashboard queries."""
import datetime
import textwrap
from typing import List
from typing import Text

import pandas as pd
from psycopg2 import sql

from exit_speed.dashboard import db_conn


def GetTracks() -> pd.DataFrame:
  select_statement = textwrap.dedent("""
  SELECT DISTINCT track
  FROM sessions
  """)
  conn = db_conn.POOL.connect()
  return pd.io.sql.read_sql(select_statement, conn)['track']


def GetSessions() -> pd.DataFrame:
  select_statement = textwrap.dedent("""
  SELECT
    TO_CHAR((duration_ms || 'millisecond')::interval, 'MI:SS:MS') AS lap_time,
    TO_CHAR(sessions.time AT TIME ZONE 'PDT', 'YYYY-MM-DD HH:MI:SS') as session_time,
    laps.id,
    laps.number AS lap_number,
    track,
    (count(points.time)::float / (duration_ms::float / 1000.0)) as points_per_second
  FROM laps
  JOIN points ON laps.id=points.lap_id
  JOIN sessions ON laps.session_id=sessions.id
  WHERE duration_ms IS NOT null
  GROUP BY sessions.id, track, sessions.time, laps.id, laps.number, lap_time, laps.duration_ms
  """)
  conn = db_conn.POOL.connect()
  return pd.io.sql.read_sql(select_statement, conn)


def GetTimeDelta(lap_ids: List[int]) -> pd.DataFrame:
  select_statement = textwrap.dedent("""
    SELECT
      a.elapsed_distance_m,
      b.elapsed_duration_ms - a.elapsed_duration_ms AS time_delta,
      b.number AS lap_number
    FROM (
      SELECT
        elapsed_duration_ms,
        ROUND(CAST(elapsed_distance_m AS numeric), 0) as elapsed_distance_m
      FROM points
      JOIN laps ON laps.id=points.lap_id
      WHERE lap_id = %(lap_id_a)s
    ) AS a INNER JOIN (
      SELECT
        elapsed_duration_ms,
        ROUND(CAST(elapsed_distance_m AS numeric), 0) as elapsed_distance_m,
        number
      FROM points
      JOIN laps ON laps.id=points.lap_id
      WHERE lap_id = %(lap_id_b)s
      ) as b
    ON a.elapsed_distance_m = b.elapsed_distance_m
    """)
  lap_id_a = lap_ids[0]
  lap_dfs = []
  for lap_id in lap_ids[1:]:
    df = pd.io.sql.read_sql(
        select_statement,
        db_conn.POOL.connect(),
        params={'lap_id_a': str(lap_id_a),
                'lap_id_b': str(lap_id)})
    df['lap_id'] = lap_id
    lap_dfs.append(df)
  combined_df = pd.concat(lap_dfs)
  combined_df.sort_values(by='elapsed_distance_m', inplace=True)
  return combined_df


def GetPointsColumns() -> List[Text]:
  select_statement = textwrap.dedent("""
  SELECT column_name
  FROM information_schema.columns
  WHERE table_name = 'points'
  """)
  conn = db_conn.POOL.connect()
  resp = conn.execute(select_statement)
  columns = [row[0] for row in resp.fetchall()]
  columns.remove('lat')
  columns.remove('lon')
  return columns


def GetLapsData(lap_ids: List[int], point_values: List[Text]) -> pd.DataFrame:
  all_columns = GetPointsColumns()
  # Only select columns that map to point_values.
  columns = set(point_values).intersection(set(all_columns))
  # Columns used for graph labels and should always be included.
  columns.update(['elapsed_distance_m', 'number', 'lap_id'])
  if ('front_brake_pressure_percentage' in point_values or
      'rear_brake_pressure_percentage' in point_values):
    columns.update(['front_brake_pressure_voltage',
                    'rear_brake_pressure_voltage'])
  if 'gsum' in point_values:
    columns.update(['accelerometer_x', 'accelerometer_y', 'accelerometer_z'])
  if 'racing_line' in point_values:
    columns.update(['lat', 'lon'])
  select_statement = textwrap.dedent("""
    SELECT {columns}
    FROM points
    JOIN laps ON points.lap_id = laps.id
    WHERE lap_id IN %(lap_ids)s
    """)
  query = sql.SQL(select_statement).format(
      columns=sql.SQL(',').join(
          [sql.Identifier(col) for col in columns]))
  raw_conn = db_conn.POOL.raw_connection()
  df = pd.io.sql.read_sql(
      query.as_string(raw_conn.cursor()),
      db_conn.POOL.connect(),
      params={'lap_ids': tuple(str(lap_id) for lap_id in lap_ids)})
  df.sort_values(by='elapsed_distance_m', inplace=True)
  df['front_brake_pressure_percentage'] = (
    df['front_brake_pressure_voltage'] /
    df['front_brake_pressure_voltage'].max())
  df['rear_brake_pressure_percentage'] = (
    df['rear_brake_pressure_voltage'] /
    df['rear_brake_pressure_voltage'].max())
  df['gsum'] = df['accelerometer_x'].abs() + df['accelerometer_y'].abs()
  df.rename(columns={'number': 'lap_number'}, inplace=True)
  return df


def GetLiveData(start_time: datetime.datetime, point_values: List[Text]):
  all_columns = GetPointsColumns()
  # Only select columns that map to point_values.
  columns = set(point_values).intersection(set(all_columns))
  # Columns used for graph labels and should always be included.
  columns.update(['time', 'number', 'lap_id'])
  select_statement = textwrap.dedent("""
    SELECT {columns}
    FROM points
    JOIN laps ON points.lap_id = laps.id
    WHERE
      time > %(start_time)s
    """)
  query = sql.SQL(select_statement).format(
      columns=sql.SQL(',').join(
          [sql.Identifier(col) for col in columns]))
  raw_conn = db_conn.POOL.raw_connection()
  df = pd.io.sql.read_sql(
      query.as_string(raw_conn.cursor()),
      db_conn.POOL.connect(),
      params={'start_time': start_time})
  df.sort_values(by='time', inplace=True)
  df.rename(columns={'number': 'lap_number'}, inplace=True)
  return df



#!/usr/bin/python3
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""DataLogger unittest."""
import os
import tempfile
import unittest

from absl.testing import absltest

from exit_speed import data_logger
from exit_speed import gps_pb2


class TestDataLogger(unittest.TestCase):
  """DataLogger unittest."""

  def setUp(self):
    super().setUp()
    _, self.file_path = tempfile.mkstemp(suffix='_1.data')
    self.point = gps_pb2.Point()
    self.point.alt = 1
    self.point.speed = 1
    self.point.lat = 45.69545832462609
    self.point.lon = -121.52551179751754
    self.point.tps_voltage = 2
    self.point.water_temp_voltage = 3
    self.point.oil_pressure_voltage = 4
    self.point.rpm = 1000
    self.point.afr = 14.7
    self.point.fuel_level_voltage = 5

  def testSetFilePath(self):
    _, prefix = tempfile.mkstemp()
    logger = data_logger.Logger(prefix)
    expected = os.path.join('%s_1.data' % prefix)
    self.assertEqual(expected , logger.file_path)

  def testGetFile(self):
    _, prefix = tempfile.mkstemp()
    logger = data_logger.Logger(prefix)
    expected = os.path.join('%s_1.data' % prefix)
    self.assertEqual(expected , logger.file_path)
    logger.GetFile(256)
    expected = os.path.join('%s_2.data' % prefix)
    self.assertEqual(expected , logger.file_path)

  def testWriteAndReadBack(self):
    logger = data_logger.Logger(self.file_path)
    logger.WriteProto(self.point)
    logger.WriteProto(self.point)
    logger.current_file.flush()
    file_points = []
    for file_point in logger.ReadProtos():
      file_points.append(file_point)
    expected = [self.point, self.point]
    self.assertEqual(expected, file_points)

  def testShortWrite(self):
    logger = data_logger.Logger(self.file_path)
    logger.WriteProto(self.point)
    logger.WriteProto(self.point)
    logger.current_file.flush()
    with open(logger.file_path, 'rb') as temp_file:
      contents = temp_file.read()
    with open(logger.file_path, 'wb') as temp_file:
      temp_file.write(contents[:-1])

    file_points = []
    for file_point in logger.ReadProtos():
      file_points.append(file_point)
    expected = [self.point]
    self.assertEqual(expected, file_points)

  def testZeroProtoBytes(self):
    logger = data_logger.Logger(self.file_path)
    logger.WriteProto(self.point)
    logger.current_file.flush()
    with open(logger.file_path, 'ab') as temp_file:
      temp_file.write((logger.current_proto_len).to_bytes(
          logger.current_proto_len, data_logger.BYTE_ORDER))

    file_points = []
    for file_point in logger.ReadProtos():
      file_points.append(file_point)
    expected = [self.point]
    self.assertEqual(expected, file_points)

  def testMidHeader(self):
    logger = data_logger.Logger(self.file_path)
    logger.WriteProto(self.point)
    logger.current_file.flush()
    with open(logger.file_path, 'ab') as temp_file:
      temp_file.write((1).to_bytes(
          logger.current_proto_len, data_logger.BYTE_ORDER))

    file_points = []
    for file_point in logger.ReadProtos():
      file_points.append(file_point)
    expected = [self.point]
    self.assertEqual(expected, file_points)

  def testReadSplitDataFiles(self):
    logger = data_logger.Logger(self.file_path)
    logger.GetFile(256)
    self.assertEqual(2, logger.current_proto_len)
    expected = os.path.join('%s_2.data' % logger.file_prefix)
    self.assertEqual(expected , logger.file_path)
    logger.WriteProto(self.point)
    logger.current_file.flush()
    file_points = []
    for file_point in logger.ReadProtos():
      file_points.append(file_point)
    expected = [self.point]
    self.assertEqual(expected, file_points)


if __name__ == '__main__':
  absltest.main()



#!/usr/bin/python3
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Library for appending protos to a file.

Based the recommendation here.
https://developers.google.com/protocol-buffers/docs/techniques
"""
import datetime
import multiprocessing
import time
from typing import Dict

from exit_speed import gps_pb2

def SleepBasedOnHertz(cycle_time: float, frequency_hz: float) -> float:
  """Calculates how to sleep to maintain the expected cycle reading.

  This is useful for sensor which are polled or have high rates.

  Args;
    cycle_time: A float of the time in the loop before the sensor reading.
    frequency_hz: The target frequency reading we're aiming for.
                  See https://docs.python.org/3/tutorial/floatingpoint.html for
                  an explanation as to why this will be slightly off.

  Returns:
    A float duration in seconds.
  """
  seconds = 1.0 / frequency_hz
  sleep = seconds - (time.time() - cycle_time)
  if sleep < 0:
    return 0
  return sleep

class SensorBase(object):
  """Base class for sensor processes."""

  def __init__(
      self,
      config: Dict,
      point_queue: multiprocessing.Queue,
      start_process: bool=True):
    self.config = config
    self._point_queue = point_queue
    self.stop_process_signal = multiprocessing.Value('b', False)
    if start_process:
      self._process = multiprocessing.Process(
          target=self.Loop,
          daemon=True)
      self._process.start()

  def StopProcess(self):
    """Sets the stop process signal."""
    self.stop_process_signal.value = True

  def Join(self):
    """Sends the stop process signal and joins the process until it finishes."""
    self.StopProcess()
    self._process.join()

  def AddPointToQueue(self, point: gps_pb2.Point):
    point.time.FromDatetime(datetime.datetime.now())
    self._point_queue.put(point.SerializeToString())

  def Loop(self):
    raise NotImplementedError('Subclasses should override this method.')



#!/usr/bin/python3
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Unitests for gyroscope.py"""
import multiprocessing
import sys
import unittest

import fake_rpi
import mock
from absl.testing import absltest
# pylint: disable=wrong-import-position
# Fixes dotstar import on Travis.
sys.modules['RPi'] = fake_rpi.RPi     # Fake RPi
sys.modules['RPi.GPIO'] = fake_rpi.RPi.GPIO # Fake GPIO
sys.modules['smbus'] = fake_rpi.smbus # Fake smbus (I2C)
import adafruit_platformdetect
with mock.patch.object(adafruit_platformdetect, 'Detector') as mock_detector:
  mock_detector.chip.id.return_value = 'BCM2XXX'
  import adafruit_fxas21002c
  import busio
  from exit_speed import gyroscope
# pylint: enable=wrong-import-position


class TestGyroscope(unittest.TestCase):
  """Gyroscope unittests."""

  def setUp(self):
    super().setUp()
    with mock.patch.object(busio, 'I2C'):
      with mock.patch.object(adafruit_fxas21002c, 'FXAS21002C'):
        self.gyroscope = gyroscope.Gyroscope()

  def testGetRotationalValues(self):
    self.gyroscope.sensor.gyroscope = (
        0.024816400301794373, -0.27052603405912107, -0.9467047653591117)
    expected = (1.421875, -15.5, -54.2421875)
    self.assertTupleEqual(expected, self.gyroscope.GetRotationalValues())

  def testGryoscopeProcessLoop(self):
    self.gyroscope.sensor.gyroscope = (
        0.024816400301794373, -0.27052603405912107, -0.9467047653591117)
    with mock.patch.object(gyroscope, 'Gyroscope') as mock_gyro:
      mock_gyro.return_value = self.gyroscope
      config = {'gyroscope': {'frequency_hz': 10}}
      point_queue = multiprocessing.Queue()
      proc = gyroscope.GyroscopeProcess(config, point_queue)
      while point_queue.empty():
        pass
      proc.Join()
      self.assertGreaterEqual(point_queue.qsize(), 0)


if __name__ == '__main__':
  absltest.main()
